from panax import route
from panax.utils.jwt_helper import jwt_encode
from models import Users


@route('users', 'login', ["POST"])
def users_login(request):

    username = request.POST.get('username', '')
    password = request.POST.get('password', '')

    if not username:
        return {"code": 400, "msg": "请输入用户名!"}

    if not password:
        return {"code": 400, "msg": "请输入密码!"}

    user = Users.get_or_none(Users.username == username)
    if not user:
        return {"code": 400, "msg": "用户名或密码错误!"}

    if user and user.verify_password(password):
        token = jwt_encode(user)
        return {
            "code": 200,
            "msg": "登录成功!",
            "data": {
                "id": user.id,
                "name": user.name,
                "username": user.username,
                "role": user.role,
                "token": token
            }
        }
    else:
        return {"code": 400, "msg": "用户名或密码错误!"}



# 全局配置


APP_SETTING = {
    # 数据库链接
    "db": {
        "host": "127.0.0.1",
        "port": 3306,
        "user": "root",
        "passwd": "",
        "database": ""
    },
    "static": "",
    # jwt 配置
    "jwt": {
        # jwt中 包含的用户列, 至少包含["id", "username", "name", "role"]，可以按需增加其他列
        "column": ["id", "username", "name", "role"],
        # jwt加密解密使用的密钥串
        "secret": "123!@#"
    },
    "file": {
        # 配置文件上传接口 可以上传的文件类型，调用方式：/api/file/image, /api/file/video
        "image": ["jpg", "jpeg", "bmp", "gif", "png"],
        "video": ["mp4"],
        "doc": ["doc", "docx"],
        "pdf": ["pdf"],
        "txt": ["txt"],
    },
    "request": {
        "secret": True,
        "key": {
            "public": b'-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEApTE9r3m0xUhm+i5JsONo\nVNVax5IKdIhLIei2U+7BfwQECPV2UpV/L7CAzq9hBJKm6wjHRRm1avEo2YjqvYjT\niCs1NG85UjaTb42iSbLCzGxdB/S6FFWk0c/i4VT8PcNdh5Dg5U/p8chp5WYlPYjk\n9WPz66z6Imf1TW9NoShwYYGFqeoZTS6uz7oU93kpOFWRLgPizysamF4AWbD7dosT\nZpYjHJEk5yqvFC/wlgNGpBMShvKcUa+ogg7VYcJu3HbLggAqrSuCrUp/ZX9F2gyR\nmNbiOyvC5QXdz7lpFqMWu7ZEJeGF+SVycknWb2O8OCzk0Padq0CNXUjtdzHy8rV/\n/QIDAQAB\n-----END PUBLIC KEY-----',
            "private": b'-----BEGIN RSA PRIVATE KEY-----\nMIIEpQIBAAKCAQEApTE9r3m0xUhm+i5JsONoVNVax5IKdIhLIei2U+7BfwQECPV2\nUpV/L7CAzq9hBJKm6wjHRRm1avEo2YjqvYjTiCs1NG85UjaTb42iSbLCzGxdB/S6\nFFWk0c/i4VT8PcNdh5Dg5U/p8chp5WYlPYjk9WPz66z6Imf1TW9NoShwYYGFqeoZ\nTS6uz7oU93kpOFWRLgPizysamF4AWbD7dosTZpYjHJEk5yqvFC/wlgNGpBMShvKc\nUa+ogg7VYcJu3HbLggAqrSuCrUp/ZX9F2gyRmNbiOyvC5QXdz7lpFqMWu7ZEJeGF\n+SVycknWb2O8OCzk0Padq0CNXUjtdzHy8rV//QIDAQABAoIBAAajz00vmOzKH3uE\nY0llJqMsTUES1XFALjbvde6zPNtYY7ohda9Vm35k+5RzN7TZmTR+zpWwNhrVlTUD\nPB5AsvH4/KM+KkFr+Kql+IU6Vnqli2q/s0AQfjbBwqaYhigMY4TItcV53BcwHVOP\ncsVBrKRRb3NEaWx7HcDOTcJB0ByXKd2wwxLxcnykMxPtlNJXuYrAYX96K/RZozei\n9l8BbDznbp26y55XFgENz7ON7KigLFQl5W+EvClTrlXpPh2t8pfnoW9CORgrDvMB\nDIp2xfnNtL0GgrZ7NS6IkayDX7Fzj1+qkfH8qTt8luOlWZsRQMCusY2x8XSbjsY5\nC5d00IkCgYEAvlnaL43nUEp4TTu1VDYGnKkvS0uyorR8RbnQUOG8OY/v3MWbMvU0\n9m+vcj+nFIpXl8YfrgdL7oPNYH+QuieI3JiGMb+u/RgX1C/lPgyzKuXOICm0CdmG\nV9kWVh/4TPMJdDr8ORmN65YgQQjbGTVASVSupUrkHE0HZJbSAV2bLfUCgYEA3iof\n7BuyOhfceLyl/1bBJ4qYjuDSMpkE1EcuoRv1bry1uYIX8vPPi0s8B1oQqkKpeyjJ\nQM5L8r234W0beRfzeGnndnjzXVYD44tQ2ToKU2CIq0G341HRCuiLror8TabU0b30\nbOnVjkevx2ChihFxOpPgi9wlbCBpPqa3FbWefOkCgYEAp3+fe/sNkdyF8chZrol/\n1fkWMdahkYTqWdzBT3rjy286LXYBNJ0LBtOOeiVC29NEcryXgebPzUlTvtdhvj0t\ntJfOdhZrYDEEPuYKfkhknJntgOXRlF1CVyki+5RURToTlqCU85XmCWloZnHpgkwW\njrCioba99Z1epQgGMcdx1sUCgYEAkCtj9dNvUDWl1BAP5OhGhkNxht6saTtVn+/l\neZVsKwV5JD4hDn9bgrg650ZyJBsIYqzwFQcK0CZ/imFEf8ukKtMG6qHIxBwKgAIr\nYBxDESJG7mPCUlkIv/xykL0Ox9FosAeF85u3AxLQJiE2EyQIh4vpHrmo60cJJSgJ\nDaSDiekCgYEAgbrrmbBzcYmRfu0vx8cdAYeIT97qQPnMFsN0VMJGXOpYnVG0acA7\nrQ+a05wrmSRGqyRcGhaAAo4SEEeZ+isAAkkfL5k3+8ANM2MWz4AEUitDVoN03lqP\ndXRAk8Tgn2xVnpo3crbGYhSDjMYnlIuNHcspGkCCwxX+efwu0WyV190=\n-----END RSA PRIVATE KEY-----'
        }
    }
}

# anymore 所有用户，包含匿名用户
# all 所有用户（登录）
# 角色
# 用户字段__表字段

# 权限配置
API_PERMISSION = {
    "__default": {
        "list": ["all"],
        "get": ["all"],
        "post": ["all"],
        "put": ["all"],
        "delete": ["all"],
        "drop": [],
        "__other": []
    },
    "file": {
        "image": ["anymore"],
        "video": ["anymore"],
        "doc": ["anymore"],
        "pdf": ["anymore"],
        "txt": ["anymore"],
    },
    "users": {
        "post": ["admin", "teacher"],
        "put": ["admin", "teacher", "id__id"],
        "delete": ["admin"],
        "login": ["anymore"]
    },
    # 权限示例，并没有创建experiment表
    "experiment": {
        "post": ["admin", "teacher"],
        # 管理员、 教师、 负责人、 所属组织的用户、 教学团队中的用户
        "put": ["admin", "teacher", "id__master_id", "org_id__org_id", "org_id__team_id"],
        "delete": ["admin", ],
        "pub": ["admin", ],
        "cancel": ["admin", ]
    }
}



import datetime
import jwt
import os, sys


sys.path.append(os.getcwd())
try:
    from config import APP_SETTING
except Exception:
    APP_SETTING = {}

def jwt_encode(user):
    # 把需要用来做权限校验的字段 都加入token中
    # 例如：{ "id": user.id, "role": user.role， "org_id": user.org_id }
    user_dict = user.to_dict()
    data = {}
    for item in APP_SETTING["jwt"]["column"]:
        data[item] = user_dict[item] if item in user_dict else ""

    return jwt.encode(data, APP_SETTING["jwt"]["secret"], algorithm='HS256')


def jwt_decode(token):
    return jwt.decode(token, APP_SETTING["jwt"]["secret"], algorithms=['HS256'])



import sys, os

sys.path.append(os.getcwd())
try:
    from config import APP_SETTING
except Exception:
    APP_SETTING = {}

import base64
from Crypto.Cipher import PKCS1_v1_5
from Crypto import Random
from Crypto.PublicKey import RSA


public_key = APP_SETTING["key"]["public"] if "key" in APP_SETTING else ""
private_key = APP_SETTING["key"]["private"] if "key" in APP_SETTING else ""


# ------------------------生成密钥对------------------------
def create_rsa_pair(is_save=False):
    '''
    创建rsa公钥私钥对
    :param is_save: default:False
    :return: public_key, private_key
    '''
    f = RSA.generate(2048)
    private_key = f.exportKey("PEM")  # 生成私钥
    public_key = f.publickey().exportKey()  # 生成公钥
    if is_save:
        with open("crypto_private_key.pem", "wb") as f:
            f.write(private_key)
        with open("crypto_public_key.pem", "wb") as f:
            f.write(public_key)
    return public_key, private_key


def read_public_key(file_path="crypto_public_key.pem") -> bytes:
    with open(file_path, "rb") as x:
        b = x.read()
        return b


def read_private_key(file_path="crypto_private_key.pem") -> bytes:
    with open(file_path, "rb") as x:
        b = x.read()
        return b


# ------------------------加密------------------------
def encryption(text: str):
    # 字符串指定编码（转为bytes）
    text = text.encode('utf-8')
    # 构建公钥对象
    cipher_public = PKCS1_v1_5.new(RSA.importKey(public_key))
    # 加密（bytes）
    text_encrypted = cipher_public.encrypt(text)
    # base64编码，并转为字符串
    text_encrypted_base64 = base64.b64encode(text_encrypted).decode()
    return text_encrypted_base64


# ------------------------解密------------------------
def decryption(text_encrypted_base64: str):
    # 字符串指定编码（转为bytes）
    text_encrypted_base64 = text_encrypted_base64.encode('utf-8')
    # base64解码
    text_encrypted = base64.b64decode(text_encrypted_base64)
    # 构建私钥对象
    cipher_private = PKCS1_v1_5.new(RSA.importKey(private_key))
    # 解密（bytes）
    text_decrypted = cipher_private.decrypt(text_encrypted, Random.new().read)
    # 解码为字符串
    text_decrypted = text_decrypted.decode()
    return text_decrypted


if __name__ == '__main__':
    # 生成密钥对
    # create_rsa_pair(is_save=True)
    # public_key = read_public_key()
    # private_key = read_private_key()
    public_key, private_key = create_rsa_pair(is_save=False)

    print("=== python ===")
    print(public_key)
    print(private_key)

    print("=== javascript ===")
    print(str(public_key).replace('\\n', ''))
    print(str(private_key).replace('\\n', ''))



    # python 中 直接使用生成的key，js中要去掉 \n

    # 加密
    # text = '123456'
    # text_encrypted_base64 = encryption(text)
    # print('密文：', text_encrypted_base64)

    # 解密
    # text_decrypted = decryption(text_encrypted_base64)
    # print('明文：', text_decrypted)



import json, time, datetime, sys, os, uuid

sys.path.append(os.getcwd())
try:
    from config import APP_SETTING
except Exception:
    APP_SETTING = {}

def file_file(request, operation):
    field_storage = request.FILES.get("file")

    allow_file_type = APP_SETTING["file"][operation]
    file_type = field_storage.filename.split('.')[-1]
    if str(file_type).lower() not in allow_file_type:
        return {
            "code": 400,
            "msg": "File Type Error!"
        }

    file_name = uuid.uuid4().hex + '.' + file_type

    root_path = os.getcwd()
    dt_path = datetime.datetime.now().strftime("%Y%m%d")
    full_path = os.path.join(root_path, 'static/files', dt_path)

    if not os.path.isdir(full_path):
        os.makedirs(full_path)

    with open(os.path.join(full_path, file_name), "wb") as f:
        f.write(field_storage.value)

    return {
        "code": 200,
        "data": '/static/files/' + dt_path + '/' + file_name,
        "msg": "Success"
    }


import datetime


def row_to_dict(cursor, row):
    """将返回结果转换为dict"""
    d = {}
    for idx, col in enumerate(cursor.description):
        if str(col[0]).startswith('_'):
            continue

        d[col[0]] = row[idx]
        if isinstance(row[idx], datetime.datetime):
            d[col[0]] = row[idx].strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(row[idx], datetime.date):
            d[col[0]] = row[idx].strftime('%Y-%m-%d')
        else:
            d[col[0]] = row[idx]
    return d


import argparse

import sys
import os
import shutil
sys.path.append(os.getcwd())

from panax.database import db
from  panax.utils.rsa import create_rsa_pair
try:
    from config import APP_SETTING
    import models
except:
    APP_SETTING = {}
    models = None

from peewee_migrate import Router


def init():
    print("==init==")
    old_path = os.path.abspath(__file__).replace("panax/tool.py", "panax/template")
    new_path = os.getcwd()
    shutil.copytree(old_path + "/", new_path + "/", dirs_exist_ok=True)
    print("==finished==")


def key():
    public_key, private_key = create_rsa_pair(is_save=False)

    print("=== python ===")
    print(public_key)
    print(private_key)

    print("=== javascript ===")
    print(str(public_key).replace('\\n', ''))
    print(str(private_key).replace('\\n', ''))


def migrate():
    print("==migrateing==")

    db.connect()

    router = Router(db, ignore="basemodel")
    router.create(auto=models)
    router.run()

    db.close()

    print("==migrate finished==")


def create_super():
    model = models.Users.create(**{
        "username": "admin",
        "password": "Aa123456",
        "name": "管理员",
        "role": "admin",
    })
    print("==finished==")


# def upgrade():
#     print("==upgrade==")
#     repo = os.path.join(os.getcwd(), 'db_migrate')
#     if not os.path.exists(repo):
#         print("Repo Not Found!")
#
#     api.upgrade(APP_SETTING["connection"], repo)


def main():
    parser = argparse.ArgumentParser()
    # parser.add_argument("-f", help="")
    parser.add_argument("exec", help="参数: [init, migrate, upgrade]")

    # 解析
    args = parser.parse_args()
    exec = args.exec
    # f = args.f

    if exec == "init":
        init()
    elif exec == "migrate":
        migrate()
    elif exec == "create_super":
        create_super()
    elif exec == "key":
        key()
    # elif exec == "upgrade":
    #     upgrade()
    else:
        print("参数错误")


if __name__ == '__main__':
    main()



import json, uuid, sys, os, datetime, mimetypes

sys.path.append(os.getcwd())

try:
    from config import APP_SETTING, API_PERMISSION
    from models import MODEL_MAPPING
except Exception:
    APP_SETTING = {}
    API_PERMISSION = {}
    MODEL_MAPPING = {}

from wsgiref.simple_server import make_server
from panax.request import Request
from panax.default_views.auto_view import auto_config
from panax.default_views.file_view import file_file

url_map = {}

request = Request()


def application(environ, star_response):
    request.bind(environ)

    if request.path.startswith('/api/static/'):
        file_path = os.path.join(APP_SETTING["static"], request.path.replace('/api/static/', ''))

        if not os.path.exists(file_path) or not os.path.isfile(file_path):
            star_response('200 OK', [('Content-Type', 'application/json;charset=urf-8')])
            response = json.dumps({"code": 404, "msg": "Not Found!"}).encode('utf-8')
            return [response, ]

        mimetype, encoding = mimetypes.guess_type(file_path)
        star_response('200 OK', [('Content-Type', mimetype)])
        return '' if request.method == 'HEAD' else open(file_path, 'rb')

    star_response('200 OK', [('Content-Type', 'application/json;charset=urf-8')])

    if request.POST == None:
        response = json.dumps({"code": 400, "msg": "Param Error!"}).encode('utf-8')
        return [response, ]

    request_path = request.path.replace('/api/', '')

    arr_path = str(request_path).split('/')

    if len(arr_path) != 2 and len(arr_path) != 3:
        response = json.dumps({"code": 404, "msg": "Not Found!"}).encode('utf-8')
        return [response, ]

    # 资源
    path_resource = arr_path[0]
    # 方法
    path_operation = arr_path[1]
    # 参数
    path_param = arr_path[2] if len(arr_path) == 3 else None

    # 权限判断
    permission = []
    if path_resource in API_PERMISSION:
        if path_operation in API_PERMISSION[path_resource]:
            permission = API_PERMISSION[path_resource][path_operation]
        elif path_operation in API_PERMISSION["__default"]:
            permission = API_PERMISSION["__default"][path_operation]
        else:
            permission = API_PERMISSION["__default"]["__other"]
    else:
        if path_operation in API_PERMISSION["__default"]:
            permission = API_PERMISSION["__default"][path_operation]
        else:
            permission = API_PERMISSION["__default"]["__other"]

    permission_check = False

    if len(permission) == 0:
        response = json.dumps({"code": 403, "msg": "Permission Denied"}).encode('utf-8')
        return [response, ]

    if "anymore" in permission:
        permission_check = True

    if "all" in permission and request.user and request.user["id"]:
        permission_check = True

    permission_role = []
    permission_column = []
    for item in permission:
        if "__" in item:
            permission_column.append(item)
        else:
            permission_role.append(item)

    if request.user["role"] in permission_role:
        permission_check = True

    if permission_check == False and len(permission_column) > 0 and path_param:
        permission_check_model = MODEL_MAPPING[table_name].get_or_none(Model.id == path_param)
        if not permission_check_model:
            response = json.dumps({"code": 404, "msg": "Not Found!"}).encode('utf-8')
            return [response, ]
        permission_check_model = permission_check_model.to_dict()
        for item in permission_column:
            user_column = item.split("__")[0]
            model_column = item.split("__")[1]
            if user_column in request.user and model_column in permission_check_model:
                if request.user[user_column] in permission_check_model[model_column]:
                    permission_check = True
                    break

    if not permission_check:
        response = json.dumps({"code": 403, "msg": "Permission Denied"}).encode('utf-8')
        return [response, ]

    # 请求地址 在url_map 中 已注册
    if path_resource in url_map and path_operation in url_map[path_resource]:
        if path_operation in url_map[path_resource]:
            func = url_map[path_resource][path_operation]

            if request.method not in func["method"]:
                response = json.dumps({"code": 405, "msg": "Method Not Allowed!"}).encode('utf-8')

            handle = func["handle"]
            if path_param is not None:
                result = handle(request, path_param)
            else:
                result = handle(request)

            response = json.dumps(result).encode('utf-8')
            return [response, ]
    elif path_resource == "file":
        # 文件接口
        if path_operation in APP_SETTING["file"]:
            if request.method != "POST":
                response = json.dumps({"code": 405, "msg": "Method Not Allowed!"}).encode('utf-8')
                return [response, ]

            result = file_file(request, path_operation)

            response = json.dumps(result).encode('utf-8')
            return [response, ]
        else:
            response = json.dumps({"code": 404, "msg": "Not Found!"}).encode('utf-8')
            return [response, ]
    else:
        if path_operation in auto_config:
            if request.method != "POST":
                response = json.dumps({"code": 405, "msg": "Method Not Allowed!"}).encode('utf-8')
                return [response, ]

            handle = auto_config[path_operation]

            if path_param is not None:
                result = handle(request, path_resource, path_param)
            else:
                result = handle(request, path_resource)

            response = json.dumps(result).encode('utf-8')
            return [response, ]
        else:
            response = json.dumps({"code": 404, "msg": "Not Found!"}).encode('utf-8')
            return [response, ]


def route(resource, operation, method=['POST']):
    def wrapper(handler):
        if resource not in url_map:
            url_map[resource] = {}

        url_map[resource][operation] = {
            "method": method,
            "handle": handler
        }
        return handler

    return wrapper


def run(host='127.0.0.1', port=8000):
    '''
    启动监听服务
    '''
    httpd = make_server(host, port, application)
    print('服务已启动 ...')
    print('正在监听 http://%s:%d/' % (host, port))
    print('按 Ctrl-C 退出')
    print('')
    httpd.serve_forever()



import sys, os

sys.path.append(os.getcwd())
try:
    from config import APP_SETTING
except Exception:
    APP_SETTING = {}

import cgi
import threading
from urllib.parse import parse_qs
from panax.utils.jwt_helper import jwt_decode

from panax.utils.request_helper import request_process


class Request(threading.local):
    def bind(self, environ):
        self._environ = environ

        # GET
        query_string = self._environ.get('QUERY_STRING', '')
        raw_dict = parse_qs(query_string, keep_blank_values=1)
        self._GET = {}
        for key, value in raw_dict.items():
            if len(value) == 1:
                self._GET[key] = value[0]
            else:
                self._GET[key] = value

        # POST FILES
        raw_data = cgi.FieldStorage(fp=self._environ['wsgi.input'], environ=self._environ)
        self._POST = {}
        self._FILES = {}
        if raw_data.list:
            for key in raw_data:
                if raw_data[key].filename:
                    self._FILES[key] = raw_data[key]
                elif isinstance(raw_data[key], list):
                    self._POST[key] = [v.value for v in raw_data[key]]
                else:
                    self._POST[key] = raw_data[key].value
        
        self._headers = None
        self._user = None

        if not self._FILES and APP_SETTING["request"]["secret"]:
            self._POST = request_process(self._POST)

    @property
    def path(self):
        return '/' + self._environ.get('PATH_INFO', '').lstrip('/')

    @property
    def GET(self):
        return self._GET

    @property
    def POST(self):
        return self._POST

    @property
    def FILES(self):
        return self._FILES

    @property
    def method(self):
        return self._environ.get('REQUEST_METHOD', 'GET').upper()

    @property
    def headers(self):
        if self._headers == None:
            self._headers = {}
            for key, value in dict(self._environ).items():
                if str(key).startswith("HTTP_"):
                    self._headers[str(key).replace("HTTP_", "")] = value
        return self._headers

    @property
    def user(self):
        if self._user == None:
            token = self.headers["AUTHORIZATION"] if "AUTHORIZATION" in self.headers and self.headers["AUTHORIZATION"] else ""
            if token:
                self._user = jwt_decode(token)
            else:
                self._user = {
                    "id": "",
                    "username": "anymore",
                    "name": "匿名用户",
                    "role": "anymore"
                }
                for item in APP_SETTING["jwt"]["column"]:
                    if item not in self._user:
                        self._user[item] = ""
        return self._user



import base64
import ast
from Crypto.Cipher import AES
from binascii import b2a_hex, a2b_hex


def add_to_16(text):
    if len(text.encode('utf-8')) % 16:
        add = 16 - (len(text.encode('utf-8')) % 16)
    else:
        add = 0
    text = text + ('\0' * add)
    return text.encode('utf-8')


def encrypt(text, key, iv):
    # 加密函数
    mode = AES.MODE_CBC
    text = add_to_16(text)
    cryptos = AES.new(key, mode, iv)
    cipher_text = cryptos.encrypt(text)
    # 因为AES加密后的字符串不一定是ascii字符集的，输出保存可能存在问题，所以这里转为16进制字符串
    return str(base64.b64encode(b2a_hex(cipher_text)), encoding='utf-8')


# 解密后，去掉补足的空格用strip() 去掉
def decrypt(text, key, iv):
    text = base64.b64decode(text)
    key = str(key).encode('utf-8')
    iv = str(iv).encode('utf-8')
    mode = AES.MODE_CBC
    cryptos = AES.new(key, mode, iv)
    # plain_text = cryptos.decrypt(a2b_hex(text))
    plain_text = cryptos.decrypt(text)
    # return bytes.decode(plain_text).rstrip('\0')
    data = bytes.decode(plain_text)
    return data[0: data.rfind("}") + 1]


if __name__ == '__main__':
    e = encrypt("""{"username":"test"}""")  # 加密
    d = decrypt(e)  # 解密
    print("加密:", e)
    print("解密:", d)
    # print(type(d))
    print(ast.literal_eval(d)['username'])



import hashlib
from peewee import *
from panax.database import BaseModel


# IntegerField, DecimalField, CharField, TextField, DateTimeField, DateField, BooleanField


class Users(BaseModel):
    """用户"""
    username = CharField(max_length=128, unique=True, index=True, help_text="用户名")
    _password = CharField(max_length=128)
    name = CharField(max_length=100)
    role = CharField(max_length=50)

    @property
    def password(self):
        raise AttributeError('password is not a readable attribute')

    @password.setter
    def password(self, password):
        # 建议加入混淆字符串，防止md5碰撞
        self._password = hashlib.md5(password.encode(encoding='utf-8')).hexdigest()

    def verify_password(self, password):
        # 建议加入混淆字符串，防止md5碰撞
        return hashlib.md5(password.encode(encoding='utf-8')).hexdigest() == self._password


MODEL_MAPPING = {
    "users": Users
}



import json, uuid, sys, os, datetime
sys.path.append(os.getcwd())

import peewee as pw

try:
    from config import APP_SETTING
except Exception:
    APP_SETTING = {}


db = pw.MySQLDatabase(
    host=APP_SETTING["db"]["host"] if "db" in APP_SETTING else "",
    port=APP_SETTING["db"]["port"] if "db" in APP_SETTING else "",
    user=APP_SETTING["db"]["user"] if "db" in APP_SETTING else "",
    passwd=APP_SETTING["db"]["passwd"] if "db" in APP_SETTING else "",
    database=APP_SETTING["db"]["database"] if "db" in APP_SETTING else "",
    charset='utf8'
)


def gen_id():
    return uuid.uuid4().hex


class BaseModel(pw.Model):
    id = pw.CharField(primary_key=True, unique=True, max_length=128, default=gen_id, verbose_name="主键ID")
    created = pw.DateTimeField(default=datetime.datetime.now, verbose_name="创建时间")
    is_del = pw.IntegerField(default=0)

    class Meta:
        database = db

    def to_dict(self):
        data = {}
        for k in self.__dict__['__data__'].keys():
            if str(k).startswith('_'):
                continue
            if isinstance(self.__dict__['__data__'][k], datetime.datetime):
                data[k] = self.__dict__['__data__'][k].strftime('%Y-%m-%d %H:%M:%S')
            elif isinstance(self.__dict__['__data__'][k], datetime.date):
                data[k] = self.__dict__['__data__'][k].strftime('%Y-%m-%d')
            else:
                data[k] = self.__dict__['__data__'][k]
        return data





import json
from panax.utils.rsa import encryption as rsa_en, decryption as rsa_de
from panax.utils.aes import encrypt as aes_en, decrypt as aes_de


def request_process(r):
    d = r.get('d', None)
    k = r.get('k', None)
    t = r.get('t', None)
    if not d or not k or not t:
        return {}
    key = rsa_de(k)
    iv = str(t)[5:] + str(t)[0:8]
    data = aes_de(d, key, iv)
    res = json.loads(data) if data else {}
    return res



import codecs
import os
from setuptools import setup, find_packages

# these things are needed for the README.md show on pypi (if you dont need delete it)
here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

# you need to change all these
VERSION = '0.0.58'
DESCRIPTION = 'sample wsgi server'

setup(
    name="panax",
    version=VERSION,
    author="smart_long",
    author_email="smart_long@outlook.com",
    description=DESCRIPTION,
    # 长描述内容的类型设置为markdown
    long_description_content_type="text/markdown",
    # 长描述设置为README.md的内容
    long_description=long_description,
    # 使用find_packages()自动发现项目中的所有包
    packages=find_packages(),
    # 包含的模块
    py_modules=['panax'],
    # 许可协议
    license='MIT',
    # 要安装的依赖包
    install_requires=[
        'urllib3==1.26.15',
        'Click',
        'pymysql',
        'peewee',
        'peewee-migrate',
        'PyJWT',
        'pycryptodome',
    ],
    entry_points={
        'console_scripts': [
            'panax=panax.tool:main'
        ]
    },
    # keywords=['python', 'panax', 'api','windows','mac','linux'],
    classifiers=[
        "Programming Language :: Python :: 3.8",
    ]
)


# python3 setup.py sdist bdist_wheel
# twine upload dist/*



import json, time, datetime, sys, os
# from panax.database import engine, conn, session
# from sqlalchemy import text

from panax.database import db

sys.path.append(os.getcwd())
try:
    from config.sql_template import SQL_TEMPLATE
    from models import MODEL_MAPPING
except Exception:
    SQL_TEMPLATE ={}
    MODEL_MAPPING = {}

from panax.utils.db_helper import row_to_dict


def generate_where_and_params(dict_where):
    #     """
    #     __exact 精确等于 like ‘aaa’
    #     __contains 包含 like ‘%aaa%’
    #     __gt 大于
    #     __gte 大于等于
    #     __lt 小于
    #     __lte 小于等于
    #     __in 存在于一个list范围内 (1, 2)
    #     __isnull 为null 不是'' ， 值：true， false
    #     """
    #
    #     """
    #     __year 时间或日期字段的年份
    #     __month 时间或日期字段的月份
    #     __day 时间或日期字段的日
    #     __date 时间或日期字段的日期部分
    #     __startswith 以…开头
    #     __endswith 以…结尾
    #     """

    arr_where = []
    param_where = dict()
    for key in dict_where.keys():
        if dict_where[key] != '':
            arr_k = key.split("__")
            if len(arr_k) == 1:
                arr_where.append(key + " = {" + key + "}")
                param_where[arr_k[0]] = dict_where[key]
            if len(arr_k) == 2:
                field = arr_k[0]
                operation = str(arr_k[1]).lower()
                if operation == "exact":
                    arr_where.append(field + " = {" + field + "}")
                    param_where[arr_k[0]] = dict_where[key]
                if operation == "contains":
                    arr_where.append(field + " like {" + field + "}")
                    param_where[arr_k[0]] = "%" + dict_where[key] + "%"
                if operation == "gt":
                    arr_where.append(field + " > {" + field + "}")
                    param_where[arr_k[0]] = dict_where[key]
                if operation == "gte":
                    arr_where.append(field + " >= {" + field + "}")
                    param_where[arr_k[0]] = dict_where[key]
                if operation == "lt":
                    arr_where.append(field + " < {" + field + "}")
                    param_where[arr_k[0]] = dict_where[key]
                if operation == "lte":
                    arr_where.append(field + " <= {" + field + "}")
                    param_where[arr_k[0]] = dict_where[key]
                if operation == "in" and len(dict_where[key]) > 0:
                    arr_where.append(field + " in (" + ",".join(['\'' + item + '\'' if isinstance(item, str) else str(item) for item in dict_where[key]]) + ") ")
                    # param_where[arr_k[0]] = dict_where[key]
                if operation == "isnull" and dict_where[key] == True:
                    arr_where.append("ISNULL(" + field + ")")
                    # param_where[arr_k[0]] = dict_where[key]
                if operation == "isnull" and dict_where[key] == False:
                    arr_where.append("NOT ISNULL(" + field + ")")
                    # param_where[arr_k[0]] = dict_where[key]

    return arr_where, param_where


def auto_list(request, table_name):
    select = request.POST.get('select', '*')
    where = request.POST.get('where', '{}')
    order_by = request.POST.get('order_by', '')

    page = int(request.POST.get('page', '1'))
    size = int(request.POST.get('size', '10000'))

    str_sql = "select " + select + " from " + table_name

    if table_name in SQL_TEMPLATE:
        str_sql = "select " + select + " from (" + SQL_TEMPLATE[table_name] + ") t_template "

    dict_temp = json.loads(where)
    dict_template = dict()
    dict_where = dict()

    for key in dict_temp:
        if '__template' in key:
            dict_template[str(key).replace('__template', '')] = dict_temp[key]
        else:
            dict_where[key] = dict_temp[key]

    if dict_template:
        str_sql = str_sql.format(**dict_template)

    arr_where, param_where = generate_where_and_params(dict_where)

    if len(arr_where) > 0:
        str_sql += " where " + " and ".join(arr_where)

    if order_by:
        str_sql += " order by " + order_by

    data_sql = "select * from (" + str_sql + ") t limit " + str((page - 1) * size) + "," + str(size)
    data_cursor = db.execute_sql(data_sql.format(**param_where))
    data = data_cursor.fetchall()

    count_sql = "select count(*) from (" + str_sql + ") t "
    count_cursor = db.execute_sql(count_sql.format(**param_where))
    count = count_cursor.fetchall()

    return {
        "code": 200,
        "data": [row_to_dict(data_cursor, row) for row in data],
        "total": count[0][0],
        "msg": "Success"
    }


def auto_get(request, table_name, pk):
    select = request.POST.get('select', '*')
    str_sql = "select " + select + " from " + table_name + " where id = '{id}' "

    cursor = db.execute_sql(str_sql.format(**{"id": pk}))
    row = cursor.fetchone()

    if not row:
        return {
            "code": 404,
            "msg": "Not Found"
        }

    data = row_to_dict(cursor, row)

    return {
        "code": 200,
        "data": data,
        "msg": "Success"
    }


def auto_post(request, table_name):
    Model = MODEL_MAPPING[table_name]

    model = Model.create(**request.POST)

    return {
        "code": 200,
        "msg": "Success",
        "data": model.to_dict()
    }


def auto_put(request, table_name, pk):

    Model = MODEL_MAPPING[table_name]
    model = Model.get_or_none(Model.id == pk)

    if not model:
        return {"code": 400, "msg": "参数错误！"}

    for r in request.POST.keys():
        model.__setattr__(r, request.POST[r])

    model.save()

    return {
        "code": 200,
        "msg": "Success",
        "data": model.to_dict()
    }


def auto_delete(request, table_name, pk):
    Model = MODEL_MAPPING[table_name]
    model = Model.get_or_none(Model.id == pk)

    if not model:
        return {"code": 400, "msg": "参数错误！"}

    model.is_del = 1
    model.save()

    return {
        "code": 200,
        "msg": "Success"
    }


def auto_drop(request, table_name, pk):
    Model = MODEL_MAPPING[table_name]
    model = Model.get_or_none(Model.id == pk)

    if not model:
        return {"code": 400, "msg": "参数错误！"}

    model.delete_instance()

    return {
        "code": 200,
        "msg": "Success"
    }


auto_config = {
    "list": auto_list,
    "get": auto_get,
    "post": auto_post,
    "put": auto_put,
    "delete": auto_delete,
    "drop": auto_drop
}
